# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseReducedPipelineStep(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'description': 'str',
        'external_service_spec': 'ResponsePipelineStepExternalServiceSpec',
        'id': 'int',
        'if_spec': 'ResponsePipelineStepIfSpec',
        'jupyter_visualization_spec': 'ResponseReducedPipelineStepJupyterVisualizationSpec',
        'manual_input_spec': 'ResponsePipelineStepManualInputSpec',
        'run_spec': 'ResponseReducedPipelineStepRunSpec',
        'step_key': 'str',
        'title': 'str',
        'type': 'str'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'description': 'description',
        'external_service_spec': 'external_service_spec',
        'id': 'id',
        'if_spec': 'if_spec',
        'jupyter_visualization_spec': 'jupyter_visualization_spec',
        'manual_input_spec': 'manual_input_spec',
        'run_spec': 'run_spec',
        'step_key': 'step_key',
        'title': 'title',
        'type': 'type'
    }

    def __init__(self, created_dt=None, description=None, external_service_spec=None, id=None, if_spec=None, jupyter_visualization_spec=None, manual_input_spec=None, run_spec=None, step_key=None, title=None, type=None, local_vars_configuration=None):  # noqa: E501
        """ResponseReducedPipelineStep - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._description = None
        self._external_service_spec = None
        self._id = None
        self._if_spec = None
        self._jupyter_visualization_spec = None
        self._manual_input_spec = None
        self._run_spec = None
        self._step_key = None
        self._title = None
        self._type = None
        self.discriminator = None

        self.created_dt = created_dt
        if description is not None:
            self.description = description
        if external_service_spec is not None:
            self.external_service_spec = external_service_spec
        self.id = id
        if if_spec is not None:
            self.if_spec = if_spec
        if jupyter_visualization_spec is not None:
            self.jupyter_visualization_spec = jupyter_visualization_spec
        if manual_input_spec is not None:
            self.manual_input_spec = manual_input_spec
        if run_spec is not None:
            self.run_spec = run_spec
        self.step_key = step_key
        self.title = title
        self.type = type

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The created_dt of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseReducedPipelineStep.


        :param created_dt: The created_dt of this ResponseReducedPipelineStep.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The description of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseReducedPipelineStep.


        :param description: The description of this ResponseReducedPipelineStep.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def external_service_spec(self):
        """Gets the external_service_spec of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The external_service_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: ResponsePipelineStepExternalServiceSpec
        """
        return self._external_service_spec

    @external_service_spec.setter
    def external_service_spec(self, external_service_spec):
        """Sets the external_service_spec of this ResponseReducedPipelineStep.


        :param external_service_spec: The external_service_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :type external_service_spec: ResponsePipelineStepExternalServiceSpec
        """

        self._external_service_spec = external_service_spec

    @property
    def id(self):
        """Gets the id of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The id of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseReducedPipelineStep.


        :param id: The id of this ResponseReducedPipelineStep.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def if_spec(self):
        """Gets the if_spec of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The if_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: ResponsePipelineStepIfSpec
        """
        return self._if_spec

    @if_spec.setter
    def if_spec(self, if_spec):
        """Sets the if_spec of this ResponseReducedPipelineStep.


        :param if_spec: The if_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :type if_spec: ResponsePipelineStepIfSpec
        """

        self._if_spec = if_spec

    @property
    def jupyter_visualization_spec(self):
        """Gets the jupyter_visualization_spec of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The jupyter_visualization_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: ResponseReducedPipelineStepJupyterVisualizationSpec
        """
        return self._jupyter_visualization_spec

    @jupyter_visualization_spec.setter
    def jupyter_visualization_spec(self, jupyter_visualization_spec):
        """Sets the jupyter_visualization_spec of this ResponseReducedPipelineStep.


        :param jupyter_visualization_spec: The jupyter_visualization_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :type jupyter_visualization_spec: ResponseReducedPipelineStepJupyterVisualizationSpec
        """

        self._jupyter_visualization_spec = jupyter_visualization_spec

    @property
    def manual_input_spec(self):
        """Gets the manual_input_spec of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The manual_input_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: ResponsePipelineStepManualInputSpec
        """
        return self._manual_input_spec

    @manual_input_spec.setter
    def manual_input_spec(self, manual_input_spec):
        """Sets the manual_input_spec of this ResponseReducedPipelineStep.


        :param manual_input_spec: The manual_input_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :type manual_input_spec: ResponsePipelineStepManualInputSpec
        """

        self._manual_input_spec = manual_input_spec

    @property
    def run_spec(self):
        """Gets the run_spec of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The run_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: ResponseReducedPipelineStepRunSpec
        """
        return self._run_spec

    @run_spec.setter
    def run_spec(self, run_spec):
        """Sets the run_spec of this ResponseReducedPipelineStep.


        :param run_spec: The run_spec of this ResponseReducedPipelineStep.  # noqa: E501
        :type run_spec: ResponseReducedPipelineStepRunSpec
        """

        self._run_spec = run_spec

    @property
    def step_key(self):
        """Gets the step_key of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The step_key of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: str
        """
        return self._step_key

    @step_key.setter
    def step_key(self, step_key):
        """Sets the step_key of this ResponseReducedPipelineStep.


        :param step_key: The step_key of this ResponseReducedPipelineStep.  # noqa: E501
        :type step_key: str
        """
        if self.local_vars_configuration.client_side_validation and step_key is None:  # noqa: E501
            raise ValueError("Invalid value for `step_key`, must not be `None`")  # noqa: E501

        self._step_key = step_key

    @property
    def title(self):
        """Gets the title of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The title of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ResponseReducedPipelineStep.


        :param title: The title of this ResponseReducedPipelineStep.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    @property
    def type(self):
        """Gets the type of this ResponseReducedPipelineStep.  # noqa: E501


        :return: The type of this ResponseReducedPipelineStep.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ResponseReducedPipelineStep.


        :param type: The type of this ResponseReducedPipelineStep.  # noqa: E501
        :type type: str
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseReducedPipelineStep):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseReducedPipelineStep):
            return True

        return self.to_dict() != other.to_dict()
