# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmOrganization(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'access_control_config': 'dict[str, object]',
        'additional_seats': 'int',
        'auto_top_up_charge_amount': 'float',
        'auto_top_up_enabled': 'bool',
        'auto_top_up_trigger_amount': 'float',
        'aws_external_id': 'str',
        'billing_plan': 'str',
        'created_dt': 'datetime',
        'credit_balance': 'float',
        'description': 'str',
        'display_name': 'str',
        'edges': 'OrmOrganizationEdges',
        'id': 'int',
        'immutable_slug': 'str',
        'name': 'str',
        'on_demand_discount_rate': 'float',
        'on_premise_discount_rate': 'float',
        'organization_default_storage': 'int',
        'organization_default_volume': 'int',
        'organization_pricing_plan': 'int',
        'organization_primary_owner': 'int',
        'owner_id': 'int',
        'show_tutorial': 'bool',
        'status': 'bool',
        'stripe_customer_id': 'str',
        'stripe_subscription_id': 'str',
        'stripe_subscription_items': 'dict[str, object]',
        'updated_dt': 'datetime',
        'user_discount_rate': 'float'
    }

    attribute_map = {
        'access_control_config': 'access_control_config',
        'additional_seats': 'additional_seats',
        'auto_top_up_charge_amount': 'auto_top_up_charge_amount',
        'auto_top_up_enabled': 'auto_top_up_enabled',
        'auto_top_up_trigger_amount': 'auto_top_up_trigger_amount',
        'aws_external_id': 'aws_external_id',
        'billing_plan': 'billing_plan',
        'created_dt': 'created_dt',
        'credit_balance': 'credit_balance',
        'description': 'description',
        'display_name': 'display_name',
        'edges': 'edges',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'name': 'name',
        'on_demand_discount_rate': 'on_demand_discount_rate',
        'on_premise_discount_rate': 'on_premise_discount_rate',
        'organization_default_storage': 'organization_default_storage',
        'organization_default_volume': 'organization_default_volume',
        'organization_pricing_plan': 'organization_pricing_plan',
        'organization_primary_owner': 'organization_primary_owner',
        'owner_id': 'owner_id',
        'show_tutorial': 'show_tutorial',
        'status': 'status',
        'stripe_customer_id': 'stripe_customer_id',
        'stripe_subscription_id': 'stripe_subscription_id',
        'stripe_subscription_items': 'stripe_subscription_items',
        'updated_dt': 'updated_dt',
        'user_discount_rate': 'user_discount_rate'
    }

    def __init__(self, access_control_config=None, additional_seats=None, auto_top_up_charge_amount=None, auto_top_up_enabled=None, auto_top_up_trigger_amount=None, aws_external_id=None, billing_plan=None, created_dt=None, credit_balance=None, description=None, display_name=None, edges=None, id=None, immutable_slug=None, name=None, on_demand_discount_rate=None, on_premise_discount_rate=None, organization_default_storage=None, organization_default_volume=None, organization_pricing_plan=None, organization_primary_owner=None, owner_id=None, show_tutorial=None, status=None, stripe_customer_id=None, stripe_subscription_id=None, stripe_subscription_items=None, updated_dt=None, user_discount_rate=None, local_vars_configuration=None):  # noqa: E501
        """OrmOrganization - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._access_control_config = None
        self._additional_seats = None
        self._auto_top_up_charge_amount = None
        self._auto_top_up_enabled = None
        self._auto_top_up_trigger_amount = None
        self._aws_external_id = None
        self._billing_plan = None
        self._created_dt = None
        self._credit_balance = None
        self._description = None
        self._display_name = None
        self._edges = None
        self._id = None
        self._immutable_slug = None
        self._name = None
        self._on_demand_discount_rate = None
        self._on_premise_discount_rate = None
        self._organization_default_storage = None
        self._organization_default_volume = None
        self._organization_pricing_plan = None
        self._organization_primary_owner = None
        self._owner_id = None
        self._show_tutorial = None
        self._status = None
        self._stripe_customer_id = None
        self._stripe_subscription_id = None
        self._stripe_subscription_items = None
        self._updated_dt = None
        self._user_discount_rate = None
        self.discriminator = None

        if access_control_config is not None:
            self.access_control_config = access_control_config
        if additional_seats is not None:
            self.additional_seats = additional_seats
        if auto_top_up_charge_amount is not None:
            self.auto_top_up_charge_amount = auto_top_up_charge_amount
        if auto_top_up_enabled is not None:
            self.auto_top_up_enabled = auto_top_up_enabled
        if auto_top_up_trigger_amount is not None:
            self.auto_top_up_trigger_amount = auto_top_up_trigger_amount
        if aws_external_id is not None:
            self.aws_external_id = aws_external_id
        if billing_plan is not None:
            self.billing_plan = billing_plan
        if created_dt is not None:
            self.created_dt = created_dt
        if credit_balance is not None:
            self.credit_balance = credit_balance
        self.description = description
        if display_name is not None:
            self.display_name = display_name
        if edges is not None:
            self.edges = edges
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if name is not None:
            self.name = name
        if on_demand_discount_rate is not None:
            self.on_demand_discount_rate = on_demand_discount_rate
        if on_premise_discount_rate is not None:
            self.on_premise_discount_rate = on_premise_discount_rate
        self.organization_default_storage = organization_default_storage
        self.organization_default_volume = organization_default_volume
        if organization_pricing_plan is not None:
            self.organization_pricing_plan = organization_pricing_plan
        if organization_primary_owner is not None:
            self.organization_primary_owner = organization_primary_owner
        self.owner_id = owner_id
        if show_tutorial is not None:
            self.show_tutorial = show_tutorial
        if status is not None:
            self.status = status
        self.stripe_customer_id = stripe_customer_id
        self.stripe_subscription_id = stripe_subscription_id
        if stripe_subscription_items is not None:
            self.stripe_subscription_items = stripe_subscription_items
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if user_discount_rate is not None:
            self.user_discount_rate = user_discount_rate

    @property
    def access_control_config(self):
        """Gets the access_control_config of this OrmOrganization.  # noqa: E501


        :return: The access_control_config of this OrmOrganization.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._access_control_config

    @access_control_config.setter
    def access_control_config(self, access_control_config):
        """Sets the access_control_config of this OrmOrganization.


        :param access_control_config: The access_control_config of this OrmOrganization.  # noqa: E501
        :type access_control_config: dict[str, object]
        """

        self._access_control_config = access_control_config

    @property
    def additional_seats(self):
        """Gets the additional_seats of this OrmOrganization.  # noqa: E501


        :return: The additional_seats of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._additional_seats

    @additional_seats.setter
    def additional_seats(self, additional_seats):
        """Sets the additional_seats of this OrmOrganization.


        :param additional_seats: The additional_seats of this OrmOrganization.  # noqa: E501
        :type additional_seats: int
        """

        self._additional_seats = additional_seats

    @property
    def auto_top_up_charge_amount(self):
        """Gets the auto_top_up_charge_amount of this OrmOrganization.  # noqa: E501


        :return: The auto_top_up_charge_amount of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._auto_top_up_charge_amount

    @auto_top_up_charge_amount.setter
    def auto_top_up_charge_amount(self, auto_top_up_charge_amount):
        """Sets the auto_top_up_charge_amount of this OrmOrganization.


        :param auto_top_up_charge_amount: The auto_top_up_charge_amount of this OrmOrganization.  # noqa: E501
        :type auto_top_up_charge_amount: float
        """

        self._auto_top_up_charge_amount = auto_top_up_charge_amount

    @property
    def auto_top_up_enabled(self):
        """Gets the auto_top_up_enabled of this OrmOrganization.  # noqa: E501


        :return: The auto_top_up_enabled of this OrmOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._auto_top_up_enabled

    @auto_top_up_enabled.setter
    def auto_top_up_enabled(self, auto_top_up_enabled):
        """Sets the auto_top_up_enabled of this OrmOrganization.


        :param auto_top_up_enabled: The auto_top_up_enabled of this OrmOrganization.  # noqa: E501
        :type auto_top_up_enabled: bool
        """

        self._auto_top_up_enabled = auto_top_up_enabled

    @property
    def auto_top_up_trigger_amount(self):
        """Gets the auto_top_up_trigger_amount of this OrmOrganization.  # noqa: E501


        :return: The auto_top_up_trigger_amount of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._auto_top_up_trigger_amount

    @auto_top_up_trigger_amount.setter
    def auto_top_up_trigger_amount(self, auto_top_up_trigger_amount):
        """Sets the auto_top_up_trigger_amount of this OrmOrganization.


        :param auto_top_up_trigger_amount: The auto_top_up_trigger_amount of this OrmOrganization.  # noqa: E501
        :type auto_top_up_trigger_amount: float
        """

        self._auto_top_up_trigger_amount = auto_top_up_trigger_amount

    @property
    def aws_external_id(self):
        """Gets the aws_external_id of this OrmOrganization.  # noqa: E501


        :return: The aws_external_id of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._aws_external_id

    @aws_external_id.setter
    def aws_external_id(self, aws_external_id):
        """Sets the aws_external_id of this OrmOrganization.


        :param aws_external_id: The aws_external_id of this OrmOrganization.  # noqa: E501
        :type aws_external_id: str
        """

        self._aws_external_id = aws_external_id

    @property
    def billing_plan(self):
        """Gets the billing_plan of this OrmOrganization.  # noqa: E501


        :return: The billing_plan of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._billing_plan

    @billing_plan.setter
    def billing_plan(self, billing_plan):
        """Sets the billing_plan of this OrmOrganization.


        :param billing_plan: The billing_plan of this OrmOrganization.  # noqa: E501
        :type billing_plan: str
        """

        self._billing_plan = billing_plan

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmOrganization.  # noqa: E501


        :return: The created_dt of this OrmOrganization.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmOrganization.


        :param created_dt: The created_dt of this OrmOrganization.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def credit_balance(self):
        """Gets the credit_balance of this OrmOrganization.  # noqa: E501


        :return: The credit_balance of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._credit_balance

    @credit_balance.setter
    def credit_balance(self, credit_balance):
        """Sets the credit_balance of this OrmOrganization.


        :param credit_balance: The credit_balance of this OrmOrganization.  # noqa: E501
        :type credit_balance: float
        """

        self._credit_balance = credit_balance

    @property
    def description(self):
        """Gets the description of this OrmOrganization.  # noqa: E501


        :return: The description of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this OrmOrganization.


        :param description: The description of this OrmOrganization.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def display_name(self):
        """Gets the display_name of this OrmOrganization.  # noqa: E501


        :return: The display_name of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name):
        """Sets the display_name of this OrmOrganization.


        :param display_name: The display_name of this OrmOrganization.  # noqa: E501
        :type display_name: str
        """

        self._display_name = display_name

    @property
    def edges(self):
        """Gets the edges of this OrmOrganization.  # noqa: E501


        :return: The edges of this OrmOrganization.  # noqa: E501
        :rtype: OrmOrganizationEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmOrganization.


        :param edges: The edges of this OrmOrganization.  # noqa: E501
        :type edges: OrmOrganizationEdges
        """

        self._edges = edges

    @property
    def id(self):
        """Gets the id of this OrmOrganization.  # noqa: E501


        :return: The id of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmOrganization.


        :param id: The id of this OrmOrganization.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmOrganization.  # noqa: E501


        :return: The immutable_slug of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmOrganization.


        :param immutable_slug: The immutable_slug of this OrmOrganization.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def name(self):
        """Gets the name of this OrmOrganization.  # noqa: E501


        :return: The name of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmOrganization.


        :param name: The name of this OrmOrganization.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def on_demand_discount_rate(self):
        """Gets the on_demand_discount_rate of this OrmOrganization.  # noqa: E501


        :return: The on_demand_discount_rate of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._on_demand_discount_rate

    @on_demand_discount_rate.setter
    def on_demand_discount_rate(self, on_demand_discount_rate):
        """Sets the on_demand_discount_rate of this OrmOrganization.


        :param on_demand_discount_rate: The on_demand_discount_rate of this OrmOrganization.  # noqa: E501
        :type on_demand_discount_rate: float
        """

        self._on_demand_discount_rate = on_demand_discount_rate

    @property
    def on_premise_discount_rate(self):
        """Gets the on_premise_discount_rate of this OrmOrganization.  # noqa: E501


        :return: The on_premise_discount_rate of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._on_premise_discount_rate

    @on_premise_discount_rate.setter
    def on_premise_discount_rate(self, on_premise_discount_rate):
        """Sets the on_premise_discount_rate of this OrmOrganization.


        :param on_premise_discount_rate: The on_premise_discount_rate of this OrmOrganization.  # noqa: E501
        :type on_premise_discount_rate: float
        """

        self._on_premise_discount_rate = on_premise_discount_rate

    @property
    def organization_default_storage(self):
        """Gets the organization_default_storage of this OrmOrganization.  # noqa: E501


        :return: The organization_default_storage of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._organization_default_storage

    @organization_default_storage.setter
    def organization_default_storage(self, organization_default_storage):
        """Sets the organization_default_storage of this OrmOrganization.


        :param organization_default_storage: The organization_default_storage of this OrmOrganization.  # noqa: E501
        :type organization_default_storage: int
        """

        self._organization_default_storage = organization_default_storage

    @property
    def organization_default_volume(self):
        """Gets the organization_default_volume of this OrmOrganization.  # noqa: E501


        :return: The organization_default_volume of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._organization_default_volume

    @organization_default_volume.setter
    def organization_default_volume(self, organization_default_volume):
        """Sets the organization_default_volume of this OrmOrganization.


        :param organization_default_volume: The organization_default_volume of this OrmOrganization.  # noqa: E501
        :type organization_default_volume: int
        """

        self._organization_default_volume = organization_default_volume

    @property
    def organization_pricing_plan(self):
        """Gets the organization_pricing_plan of this OrmOrganization.  # noqa: E501


        :return: The organization_pricing_plan of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._organization_pricing_plan

    @organization_pricing_plan.setter
    def organization_pricing_plan(self, organization_pricing_plan):
        """Sets the organization_pricing_plan of this OrmOrganization.


        :param organization_pricing_plan: The organization_pricing_plan of this OrmOrganization.  # noqa: E501
        :type organization_pricing_plan: int
        """

        self._organization_pricing_plan = organization_pricing_plan

    @property
    def organization_primary_owner(self):
        """Gets the organization_primary_owner of this OrmOrganization.  # noqa: E501


        :return: The organization_primary_owner of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._organization_primary_owner

    @organization_primary_owner.setter
    def organization_primary_owner(self, organization_primary_owner):
        """Sets the organization_primary_owner of this OrmOrganization.


        :param organization_primary_owner: The organization_primary_owner of this OrmOrganization.  # noqa: E501
        :type organization_primary_owner: int
        """

        self._organization_primary_owner = organization_primary_owner

    @property
    def owner_id(self):
        """Gets the owner_id of this OrmOrganization.  # noqa: E501


        :return: The owner_id of this OrmOrganization.  # noqa: E501
        :rtype: int
        """
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id):
        """Sets the owner_id of this OrmOrganization.


        :param owner_id: The owner_id of this OrmOrganization.  # noqa: E501
        :type owner_id: int
        """

        self._owner_id = owner_id

    @property
    def show_tutorial(self):
        """Gets the show_tutorial of this OrmOrganization.  # noqa: E501


        :return: The show_tutorial of this OrmOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._show_tutorial

    @show_tutorial.setter
    def show_tutorial(self, show_tutorial):
        """Sets the show_tutorial of this OrmOrganization.


        :param show_tutorial: The show_tutorial of this OrmOrganization.  # noqa: E501
        :type show_tutorial: bool
        """

        self._show_tutorial = show_tutorial

    @property
    def status(self):
        """Gets the status of this OrmOrganization.  # noqa: E501


        :return: The status of this OrmOrganization.  # noqa: E501
        :rtype: bool
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this OrmOrganization.


        :param status: The status of this OrmOrganization.  # noqa: E501
        :type status: bool
        """

        self._status = status

    @property
    def stripe_customer_id(self):
        """Gets the stripe_customer_id of this OrmOrganization.  # noqa: E501


        :return: The stripe_customer_id of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._stripe_customer_id

    @stripe_customer_id.setter
    def stripe_customer_id(self, stripe_customer_id):
        """Sets the stripe_customer_id of this OrmOrganization.


        :param stripe_customer_id: The stripe_customer_id of this OrmOrganization.  # noqa: E501
        :type stripe_customer_id: str
        """

        self._stripe_customer_id = stripe_customer_id

    @property
    def stripe_subscription_id(self):
        """Gets the stripe_subscription_id of this OrmOrganization.  # noqa: E501


        :return: The stripe_subscription_id of this OrmOrganization.  # noqa: E501
        :rtype: str
        """
        return self._stripe_subscription_id

    @stripe_subscription_id.setter
    def stripe_subscription_id(self, stripe_subscription_id):
        """Sets the stripe_subscription_id of this OrmOrganization.


        :param stripe_subscription_id: The stripe_subscription_id of this OrmOrganization.  # noqa: E501
        :type stripe_subscription_id: str
        """

        self._stripe_subscription_id = stripe_subscription_id

    @property
    def stripe_subscription_items(self):
        """Gets the stripe_subscription_items of this OrmOrganization.  # noqa: E501


        :return: The stripe_subscription_items of this OrmOrganization.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._stripe_subscription_items

    @stripe_subscription_items.setter
    def stripe_subscription_items(self, stripe_subscription_items):
        """Sets the stripe_subscription_items of this OrmOrganization.


        :param stripe_subscription_items: The stripe_subscription_items of this OrmOrganization.  # noqa: E501
        :type stripe_subscription_items: dict[str, object]
        """

        self._stripe_subscription_items = stripe_subscription_items

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmOrganization.  # noqa: E501


        :return: The updated_dt of this OrmOrganization.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmOrganization.


        :param updated_dt: The updated_dt of this OrmOrganization.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def user_discount_rate(self):
        """Gets the user_discount_rate of this OrmOrganization.  # noqa: E501


        :return: The user_discount_rate of this OrmOrganization.  # noqa: E501
        :rtype: float
        """
        return self._user_discount_rate

    @user_discount_rate.setter
    def user_discount_rate(self, user_discount_rate):
        """Sets the user_discount_rate of this OrmOrganization.


        :param user_discount_rate: The user_discount_rate of this OrmOrganization.  # noqa: E501
        :type user_discount_rate: float
        """

        self._user_discount_rate = user_discount_rate

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmOrganization):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmOrganization):
            return True

        return self.to_dict() != other.to_dict()
